# SPDX-FileCopyrightText: Copyright © 2023 Idiap Research Institute <contact@idiap.ch>
#
# SPDX-License-Identifier: GPL-3.0-or-later

"""Indian collection dataset for computer-aided diagnosis.

The Indian collection database has been established to foster research
in computer-aided diagnosis of pulmonary diseases with a special
focus on pulmonary tuberculosis (TB).

* Reference: [INDIAN-2013]_
* Original resolution (height x width or width x height): more than 1024 x 1024
* Split reference: [INDIAN-2013]_ with 20% of train set for the validation set
"""

import importlib.resources
import os

from ...utils.rc import load_rc
from .. import make_dataset
from ..dataset import JSONDataset
from ..loader import load_pil_baw, make_delayed

_protocols = [
    importlib.resources.files(__name__).joinpath("default.json.bz2"),
    importlib.resources.files(__name__).joinpath("fold_0.json.bz2"),
    importlib.resources.files(__name__).joinpath("fold_1.json.bz2"),
    importlib.resources.files(__name__).joinpath("fold_2.json.bz2"),
    importlib.resources.files(__name__).joinpath("fold_3.json.bz2"),
    importlib.resources.files(__name__).joinpath("fold_4.json.bz2"),
    importlib.resources.files(__name__).joinpath("fold_5.json.bz2"),
    importlib.resources.files(__name__).joinpath("fold_6.json.bz2"),
    importlib.resources.files(__name__).joinpath("fold_7.json.bz2"),
    importlib.resources.files(__name__).joinpath("fold_8.json.bz2"),
    importlib.resources.files(__name__).joinpath("fold_9.json.bz2"),
]

_datadir = load_rc().get("datadir.indian", os.path.realpath(os.curdir))


def _raw_data_loader(sample):
    return dict(
        data=load_pil_baw(os.path.join(_datadir, sample["data"])),
        label=sample["label"],
    )


def _loader(context, sample):
    # "context" is ignored in this case - database is homogeneous
    # we returned delayed samples to avoid loading all images at once
    return make_delayed(sample, _raw_data_loader)


json_dataset = JSONDataset(
    protocols=_protocols,
    fieldnames=("data", "label"),
    loader=_loader,
)
"""Indian dataset object."""


def _maker(protocol, resize_size=512, cc_size=512, RGB=False):
    from torchvision import transforms

    from ..augmentations import ElasticDeformation
    from ..image_utils import RemoveBlackBorders

    post_transforms = []
    if RGB:
        post_transforms = [
            transforms.Lambda(lambda x: x.convert("RGB")),
            transforms.ToTensor(),
        ]

    return make_dataset(
        [json_dataset.subsets(protocol)],
        [
            RemoveBlackBorders(),
            transforms.Resize(resize_size),
            transforms.CenterCrop(cc_size),
        ],
        [ElasticDeformation(p=0.8)],
        post_transforms,
    )
