# SPDX-FileCopyrightText: Copyright © 2023 Idiap Research Institute <contact@idiap.ch>
#
# SPDX-License-Identifier: GPL-3.0-or-later

import importlib.resources
import os

import PIL.Image

from torchvision.transforms.functional import center_crop, to_tensor

from ...utils.rc import load_rc
from ..datamodule import CachingDataModule
from ..image_utils import remove_black_borders
from ..split import JSONDatabaseSplit
from ..typing import RawDataLoader as _BaseRawDataLoader
from ..typing import Sample


class RawDataLoader(_BaseRawDataLoader):
    """A specialized raw-data-loader for the Shenzen dataset.

    Attributes
    ----------

    datadir
        This variable contains the base directory where the database raw data
        is stored.

    transform
        Transforms that are always applied to the loaded raw images.
    """

    datadir: str

    def __init__(self):
        self.datadir = load_rc().get(
            "datadir.shenzhen", os.path.realpath(os.curdir)
        )

    def sample(self, sample: tuple[str, int]) -> Sample:
        """Loads a single image sample from the disk.

        Parameters
        ----------

        sample:
            A tuple containing the path suffix, within the dataset root folder,
            where to find the image to be loaded, and an integer, representing the
            sample label.


        Returns
        -------

        sample
            The sample representation
        """
        # N.B.: Image.convert("L") is required to normalize grayscale back to
        # normal (instead of inverted).
        image = PIL.Image.open(os.path.join(self.datadir, sample[0])).convert(
            "L"
        )
        image = remove_black_borders(image)
        tensor = to_tensor(image)
        tensor = center_crop(tensor, min(*tensor.shape[1:]))

        # use the code below to view generated images
        # from torchvision.transforms.functional import to_pil_image
        # to_pil_image(tensor).show()
        # __import__("pdb").set_trace()

        return tensor, dict(label=sample[1], name=sample[0])  # type: ignore[arg-type]

    def label(self, sample: tuple[str, int]) -> int:
        """Loads a single image sample label from the disk.

        Parameters
        ----------

        sample:
            A tuple containing the path suffix, within the dataset root folder,
            where to find the image to be loaded, and an integer, representing the
            sample label.


        Returns
        -------

        label
            The integer label associated with the sample
        """
        return sample[1]


class DataModule(CachingDataModule):
    """Shenzhen datamodule for computer-aided diagnosis.

    The standard digital image database for Tuberculosis was created by the National
    Library of Medicine, Maryland, USA in collaboration with Shenzhen No.3 People’s
    Hospital, Guangdong Medical College, Shenzhen, China. The Chest X-rays are from
    out-patient clinics, and were captured as part of the daily routine using
    Philips DR Digital Diagnose systems.

    * Database reference: [MONTGOMERY-SHENZHEN-2014]_
    * Original resolution (height x width or width x height): 3000 x 3000 or less
    * This split:

      * Split reference: None
      * Training samples: 64% of TB and healthy CXR (including labels)
      * Validation samples: 16% of TB and healthy CXR (including labels)
      * Test samples: 20% of TB and healthy CXR (including labels)

    Data specifications:

    * Raw data input (on disk):

        * PNG images (grayscale, encoded as RGB images with "inverted" grayscale scale)
        * Variable width and height

    * Output image:

        * Transforms:

            * Load raw PNG with :py:mod:`PIL`
            * Remove black borders
            * Torch center cropping to get square image

        * Final specifications:

            * Grayscale, encoded as a single plane image, 8 bits
            * Square, with varying resolutions, depending on the input image
    """

    def __init__(self, split_filename: str):
        super().__init__(
            database_split=JSONDatabaseSplit(
                importlib.resources.files(__name__.rsplit(".", 1)[0]).joinpath(
                    split_filename
                )
            ),
            raw_data_loader=RawDataLoader(),
        )


default = DataModule("default.json.bz2")
fold_0 = DataModule("fold_0.json.bz2")
fold_1 = DataModule("fold_1.json.bz2")
fold_2 = DataModule("fold_2.json.bz2")
fold_3 = DataModule("fold_3.json.bz2")
fold_4 = DataModule("fold_4.json.bz2")
fold_5 = DataModule("fold_5.json.bz2")
fold_6 = DataModule("fold_6.json.bz2")
fold_7 = DataModule("fold_7.json.bz2")
fold_8 = DataModule("fold_8.json.bz2")
fold_9 = DataModule("fold_9.json.bz2")
